#include <curl/curl.h>
#include <iostream>
#include <vector>
#include <sstream>
#include <string>
#include "histogram.h"
#include "text.h"
#include "svg.h"

using namespace std;


struct Input {
    vector<double> numbers;
    size_t bin_count{};
    size_t interval_task{};
};

size_t
write_data(void* items, size_t item_size, size_t item_count, void* ctx) {
    size_t data_size = item_size * item_count;

    //struct Input *i = (struct Input *)ctx;

    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<const char*>(items), data_size);

    return data_size;
}



Input
input_data(istream& in, bool prompt)
{
    if (prompt == false) {                                           //prompt output
        cerr.rdbuf(NULL);
    }

    size_t number_count;
    cerr << "Enter number count: \n";
    in >> number_count;

    Input in_data_struct;

    in_data_struct.numbers.resize(number_count);
    for (size_t i = 0; i < number_count; i++) {
        cerr << "Enter number "<< i << " : \n";
        in >> in_data_struct.numbers[i];
    }

    cerr << "Enter bin count: \n";
    in >> in_data_struct.bin_count;

    cerr << "Enter interval from task: \n";
    in >> in_data_struct.interval_task;

    if ((in_data_struct.interval_task >= 2.0) && (in_data_struct.interval_task <= 9.0)) {       //scale interval size check
        return in_data_struct;
    } else {
        std::cout << "ERROR";                                           //error situation and program exit
        exit(1);
    }
}

Input
download(const string& address) {
    stringstream buffer;
    stringstream  temp;

    CURL* curl = curl_easy_init();
    if(curl) {
        CURLcode res;
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);

        curl_easy_setopt(curl, CURLOPT_HEADERFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_HEADERDATA, &temp);

        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);

        res = curl_easy_perform(curl);
        if (CURLE_OK != 0) {
                cerr << curl_easy_strerror(res);
                exit(1);
            }
        cerr << temp.str();
        curl_easy_cleanup(curl);
    }
    return input_data(buffer, false);
}

int
main(int argc, char* argv[]) {
    Input input;
    if (argc > 1) {
        input = download(argv[1]);
    } else {
        input = input_data(cin, true);
    }

    const auto bins = make_histogram(input.numbers, input.bin_count);
    show_histogram_svg(bins, input.interval_task);

    //[OUTDATED]
    //curl_global_init(CURL_GLOBAL_ALL);
    // in = input_data(cin, true);                                             //input data
    //auto bins = make_histogram(in.numbers, in.bin_count);               //calculating bin size for the histogram
    //show_histogram_svg(bins, in.interval_task);                         //histogram output in the format svg code

    return 0;
}

